#lang racket/base

(require 
  racket/system
  racket/string)

(provide (except-out (all-from-out racket/base) 
                      #%module-begin)
         (rename-out [module-begin #%module-begin])
         add-opt!
         hsplit
         split
         bind
         bind-exec
         execm
         exec
         set-padding
         terminal
         unmanage
         set-terminal!
         set-default-background
         set-background
         xresources
         enable-transparent-compositing)

(define nightshade-opts-private (list))
(define xresources-private (list))

(define default-terminal "urxvt256c-ml")

(define (xresources . lst)
  (set! xresources-private (append xresources-private lst)))

(define (bind k command)
  (add-opt! (format "bind ~a ~a" k command)))

(define (bind-exec k command)
  (add-opt! (format "bind ~a exec ~a" k command)))

(define (execm command)
  (add-opt! (format "execm ~a" command)))

(define (exec command)
  (add-opt! (format "exec ~a" command)))

(define (set-padding w x y z)
  (add-opt! (format "set padding ~a ~a ~a ~a" w x y z)))

(define (unmanage wname)
  (add-opt! (format "unmanage ~a" wname)))

(define (byte-string-wrapper str)
  (cond [(bytes? str) str]
        [(string? str) (string->bytes/locale str)]
        [(path? str)
           (string->bytes/locale
             (path->string str))]
        [else (error "dynamic-ffi: unsupported format string: " str)]))

(define (add-opt! opt)
 (set! nightshade-opts-private 
  (cons (byte-string-wrapper opt) nightshade-opts-private)))
(define (hsplit)
 (add-opt! #"hsplit"))
(define (split)
 (add-opt! #"split"))
(define (terminal)
 (exec default-terminal))

(define (set-terminal! t)
  (set! default-terminal t))


(define (enable-transparent-compositing)
 (add-opt! #"exec xcompmgr -c -f -D 5 &"))

        (define default-background #"/usr/share/backgrounds/f28/default/standard/f28.png")

(define (set-background image-path) 
 (add-opt! (bytes-append #"exec nitrogen --set-zoom-fill " (byte-string-wrapper image-path))))

(define (set-default-background)
 (set-background default-background))

(define-syntax-rule (module-begin expr ...)
 (#%module-begin
  (begin expr ...)
  (define nightshade-opts (reverse nightshade-opts-private))
  (system (format "bash -c 'xrdb -merge <(echo \"~a\") '" (string-join xresources-private "\n")))
  
  (provide nightshade-opts)))
